-- ############ --
-- Deja Minimap
-- ############ --
local thisAddOn, private = ...
local thisAddOnName = GetAddOnInfo(thisAddOn)
local version = GetAddOnMetadata(thisAddOn, "Version")
local addoninfo = 'v'..version

-- #################### --
-- SavedVariables Setup
-- #################### --
private.defaults = {}

private.defaults.mmsliderSetScale = {
	minimapScale = 0.87,
}

private.loader = CreateFrame("Frame")
	private.loader:RegisterEvent("ADDON_LOADED")
	private.loader:SetScript("OnEvent", function(self, event, ...)
		private.loader[event](self, ...) -- call event handlers
	end)

	function private.loader:ADDON_LOADED(thisAddOn)
		if not (_G[thisAddOnName.."DBPC"]) then -- first time the addon is used
			print("DejaMinimap: First time use, creating saved variables")
			_G[thisAddOnName.."DBPC"] = {} -- allocate a table to store your saved variables (the table will be saved to disk on logout)
			-- Now, load the table with defaults if needed.
			_G[thisAddOnName.."DBPC"] = private.defaults
		end
		-- Initialise the addon with the Saved Variables information (new or loaded from disk)
		private.db = _G[thisAddOnName.."DBPC"]
		self:UnregisterEvent("ADDON_LOADED")
	end

-- ################### --
-- Minimap Coordinates
-- ################### --
local coordFS = MinimapCluster.BorderTop:CreateFontString("FontString","OVERLAY","GameTooltipText")
	coordFS:SetTextColor(1, 1, 1);
	
local function coordRefresh()
	local mapID = C_Map.GetBestMapForUnit("player")
	if mapID then
		local pos = C_Map.GetPlayerMapPosition(mapID, "player")
		if (pos == nil) or (pos.x == nil) or (pos.y == nil) then
			coordFS:SetText("--, --")
			return
		end
		if pos and pos.x ~= 0 and pos.y ~= 0 then
			coordFS:SetFormattedText("%.1f, %.1f", pos.x*100, pos.y*100)
		end
	else
		coordFS:SetText("--, --")
	end
end	
	
local elapsed = 0
private.loader:SetScript("OnUpdate", function(self, e)
	local inInstance, instanceType = IsInInstance()
	if not IsInInstance(player) then
		elapsed = elapsed + e
		if elapsed >= 1 then
			coordRefresh()
			elapsed = 0
		end
	else
		coordFS:SetFormattedText("")
	end
end)

-- ############## --
-- Minimap Slider
-- ############## --
local function DMM_SetPoints() -- Function called by slash commands; skip to --Slash Setup if a reset function isn't needed.
	--Slider
	local slideValue = private.db.mmsliderSetScale.minimapScale
		MMSlider:SetValue(slideValue)
	MinimapCluster:SetScale(slideValue)
	getglobal(MMSlider:GetName() .. 'Text'):SetFormattedText("Minimap Scale = (%.2f)", (slideValue)); --Sets the "title" text (top-centre of slider).
end

local MMSlider = CreateFrame("Slider", "MMSlider", EditModeSystemSettingsDialog, "OptionsSliderTemplate")
	MMSlider:RegisterEvent("PLAYER_LOGIN")
	MMSlider:RegisterEvent("PLAYER_ENTERING_WORLD")
	MMSlider:RegisterEvent("ADDON_LOADED")
	MMSlider:SetPoint("BOTTOMLEFT", EditModeSystemSettingsDialog, "BOTTOMLEFT", 26, 26)
	MMSlider:SetPoint("BOTTOMRIGHT", EditModeSystemSettingsDialog, "BOTTOMRIGHT", -26, -26)
	MMSlider:SetFrameStrata("TOOLTIP")
	MMSlider:SetOrientation('HORIZONTAL')
	MMSlider:SetMinMaxValues(0.50, 3.0)
	MMSlider.minValue, MMSlider.maxValue = MMSlider:GetMinMaxValues() 
	MMSlider:SetValueStep(0.05)
	MMSlider:SetObeyStepOnDrag(true)

	MMSlider.tooltipText = 'Scale the minimap in increments or decrements of 5' --Creates a tooltip on mouseover.

	getglobal(MMSlider:GetName() .. 'Low'):SetText(MMSlider.minValue); --Sets the left-side slider text (default is "Low").
	getglobal(MMSlider:GetName() .. 'High'):SetText(MMSlider.maxValue); --Sets the right-side slider text (default is "High").
	
	MMSlider:SetScript("OnEvent", function(self, event, ...)
		DMM_SetPoints()
	end)

	MMSlider:SetScript("OnValueChanged", function(self, value) 
	local slideValue = MMSlider:GetValue()
		MinimapCluster:SetScale(slideValue)
		getglobal(MMSlider:GetName() .. 'Text'):SetFormattedText("Minimap Scale = (%.2f)", (slideValue)); --Sets the "title" text (top-centre of slider).
		private.db.mmsliderSetScale.minimapScale = slideValue
	end)
	
	MinimapCluster:SetScript("OnEvent", function(self, event, ...)
		local headerUnderneath = self:GetSettingValueBool(Enum.EditModeMinimapSetting.HeaderUnderneath)
		local slideValue = private.db.mmsliderSetScale
			self:SetScale(slideValue.minimapScale)
			
		if (MouseIsOver(MinimapCluster.Selection)) then
			if MinimapCluster.isSelected == true then
				EditModeSystemSettingsDialog:SetHeight(self:GetHeight() * 1.10)
			end
			MMSlider:Show()
		elseif (not (MouseIsOver(EditModeSystemSettingsDialog))) or (not (MouseIsOver(EditModeSystemSettingsDialog))) then
			MMSlider:Hide()
		end

		MinimapCluster:RegisterEvent("PLAYER_LOGIN")
		MinimapCluster:RegisterEvent("PLAYER_ENTERING_WORLD")
		MinimapCluster:RegisterEvent("ADDON_LOADED")
		MinimapCluster:RegisterEvent("GLOBAL_MOUSE_UP") -- Using UP isntead of DOWN so it checks after button clicks and dragging frames.
		MinimapCluster:SetClampRectInsets(0, 0, 0, 0)		
		
		MinimapCluster.Selection:ClearAllPoints()
		MinimapCluster.Selection:SetAllPoints(MinimapCluster)
		MinimapCluster.Selection:SetSize(MinimapCluster:GetWidth(), MinimapCluster:GetHeight())
		MinimapCluster.Selection:SetClampRectInsets(0, 0, 0, 0)
		
		Minimap:ClearAllPoints()
		Minimap:SetAllPoints(MinimapCluster.Selection)
		Minimap:SetSize(MinimapCluster:GetWidth(), MinimapCluster:GetHeight())
		Minimap:SetClampRectInsets(0, 0, 0, 0)
		-- Minimap Texture Mask
		Minimap:SetMaskTexture('Interface\\ChatFrame\\ChatFrameBackground')
		-- Compass Texture
		MinimapCompassTexture:Hide()

		MinimapBackdrop:ClearAllPoints()
		MinimapBackdrop:SetAllPoints(Minimap)
		MinimapBackdrop:SetSize(MinimapCluster:GetWidth(), MinimapCluster:GetHeight())
		MinimapBackdrop:SetClampRectInsets(0, 0, 0, 0)

		if (headerUnderneath) then
			MinimapCluster.BorderTop:ClearAllPoints()
			MinimapCluster.BorderTop:SetPoint("BOTTOM", MinimapCluster, "BOTTOM", 0, 0)
			MinimapCluster.BorderTop:SetWidth(MinimapCluster:GetWidth() - (MinimapCluster.Tracking:GetWidth() + GameTimeFrame:GetWidth()))

			MinimapCluster.Tracking:ClearAllPoints()
			MinimapCluster.Tracking:SetPoint("BOTTOMLEFT", MinimapCluster, "BOTTOMLEFT", 0, 0)

			GameTimeFrame:ClearAllPoints()
			GameTimeFrame:SetPoint("BOTTOMRIGHT", MinimapCluster, "BOTTOMRIGHT", 2, 0)

			MinimapCluster.MailFrame:ClearAllPoints()
			MinimapCluster.MailFrame:SetPoint("BOTTOMLEFT", MinimapCluster.Tracking, "TOPLEFT", 0, 0)

			Minimap.ZoomHitArea:ClearAllPoints()
			Minimap.ZoomHitArea:SetPoint("BOTTOMRIGHT", MinimapCluster, "BOTTOMRIGHT", 0, MinimapCluster.BorderTop:GetHeight())

			Minimap.ZoomOut:ClearAllPoints()
			Minimap.ZoomOut:SetPoint("CENTER", Minimap.ZoomHitArea, "BOTTOM", -Minimap.ZoomHitArea:GetHeight() / 4, Minimap.ZoomHitArea:GetHeight() / 3)

			Minimap.ZoomIn:ClearAllPoints()
			Minimap.ZoomIn:SetPoint("CENTER", Minimap.ZoomHitArea, "RIGHT", -Minimap.ZoomHitArea:GetWidth() / 3, Minimap.ZoomHitArea:GetHeight() / 4)
		
			coordFS:SetPoint("CENTER", MinimapCluster.BorderTop, "CENTER", 0, (MinimapCluster.BorderTop:GetHeight()))
		else
			MinimapCluster.BorderTop:ClearAllPoints()
			MinimapCluster.BorderTop:SetPoint("TOP", MinimapCluster, "TOP", 0, 0)
			MinimapCluster.BorderTop:SetWidth(MinimapCluster:GetWidth() - (MinimapCluster.Tracking:GetWidth() + GameTimeFrame:GetWidth()))

			MinimapCluster.Tracking:ClearAllPoints()
			MinimapCluster.Tracking:SetPoint("TOPLEFT", MinimapCluster, "TOPLEFT", 0, 0)

			GameTimeFrame:ClearAllPoints()
			GameTimeFrame:SetPoint("TOPRIGHT", MinimapCluster, "TOPRIGHT", 2, 0)

			MinimapCluster.MailFrame:ClearAllPoints()
			MinimapCluster.MailFrame:SetPoint("TOPLEFT", MinimapCluster.Tracking, "BOTTOMLEFT", 0, 0)

			Minimap.ZoomHitArea:ClearAllPoints()
			Minimap.ZoomHitArea:SetPoint("BOTTOMRIGHT", MinimapCluster, "BOTTOMRIGHT", 0, 0)

			Minimap.ZoomOut:ClearAllPoints()
			Minimap.ZoomOut:SetPoint("CENTER", Minimap.ZoomHitArea, "BOTTOM", -Minimap.ZoomHitArea:GetHeight() / 4, Minimap.ZoomHitArea:GetHeight() / 3)

			Minimap.ZoomIn:ClearAllPoints()
			Minimap.ZoomIn:SetPoint("CENTER", Minimap.ZoomHitArea, "RIGHT", -Minimap.ZoomHitArea:GetWidth() / 3, Minimap.ZoomHitArea:GetHeight() / 4)
			
			coordFS:SetPoint("CENTER", MinimapCluster.BorderTop, "CENTER", 0, -(MinimapCluster.BorderTop:GetHeight()))
		end
	end)